#! /bin/sh
exec guile-gtk -e main -s $0 $*
!#
;; Time-stamp: <1998-03-15 21:34:42 szi>
;;
;; Copyright (C) 1997, 1998 Marius Vollmer
;; 
;; Test-Gtk for Guile-Gtk
;;
;; This program should do the same like the test-gtk program, which
;; comes with Gtk.
;;

(use-modules (gtk gtk))

(define call/cc call-with-current-continuation)

(define-macro (with-return . forms)
  `(call/cc (lambda (return) ,@forms)))

(define (gtk-widget-visible? w)
  (memq 'visible (gtk-widget-flags w)))

(define (make-dialog-creator inner-creator)
  (let ((window #f))
    (lambda ()
      (if window
	  (pk (gtk-widget-flags window)))
      (cond ((not window)
	     (set! window (inner-creator))
	     (gtk-signal-connect window "destroy" 
				 (lambda () (set! window #f)))))
      (if (gtk-widget-visible? window)
	  (gtk-widget-destroy window)
	  (gtk-widget-show window)))))

(define (make-creator title inner-creator)
  (define (outer-creator)
    (let ((window (gtk-window-new 'toplevel))
	  (box1 (gtk-vbox-new #f 0))
	  (box2 (gtk-hbox-new #f 10))
	  (box3 (gtk-vbox-new #f 10))
	  (separator (gtk-hseparator-new))
	  (close (gtk-button-new-with-label "close"))
	  (buttons '()))
      (gtk-window-set-title window title)
      (gtk-container-border-width window 0)
      (gtk-container-add window box1)
      (gtk-widget-show box1)

      (gtk-box-pack-start box1 box3 #t #t 0)
      (gtk-container-border-width box3 10)
      (gtk-widget-show box3)
      (set! buttons (inner-creator box3))
	       
      (gtk-box-pack-start box1 separator #f #t 0)
      (gtk-widget-show separator)
      (gtk-container-border-width box2 10)
      (gtk-box-pack-start box1 box2 #f #t 0)
      (gtk-widget-show box2)
      (gtk-signal-connect close "clicked" 
			  (lambda () (gtk-widget-destroy window)))
      (gtk-box-pack-start box2 close #t #t 0)
      (gtk-widget-set-flags close '(can-default))
      (gtk-widget-grab-default close)
      (gtk-widget-show close)
      (for-each (lambda (bt)
		  (let ((b (gtk-button-new-with-label (car bt))))
		    ; (gtk-widget-set-flags b '(can-default))
		    (gtk-box-pack-start box2 b #t #t 0)
		    (gtk-signal-connect b "clicked" (cadr bt))
		    (gtk-widget-show b)))
		buttons)
      window))
  (make-dialog-creator outer-creator))

(define-macro (define-test sym title . forms)
  `(define ,sym (make-creator ,title 
			      (lambda (outer-box) ,@forms))))

(define-macro (define-dialog-test sym . forms)
  `(define ,sym (make-dialog-creator (lambda () ,@forms))))

(define-test create-buttons "buttons"
  (let*                  ; name      x y clicked
      ((button-template '(("button1" 0 0 1)
			  ("button2" 1 1 2)
			  ("button3" 2 2 3)
			  ("button4" 0 2 4)
			  ("button5" 2 0 5)
			  ("button6" 1 2 6)
			  ("button7" 1 0 7)
			  ("button8" 2 1 8)
			  ("button9" 0 1 0)))
	(toggle-button (lambda (b)
			 (if (gtk-widget-visible? b)
			     (gtk-widget-hide b)
			     (gtk-widget-show b))
			 (if break-iteration (break-iteration #f))))

	(table (gtk-table-new 3 3 #f))
	(buttons (map (lambda (bt)
			(gtk-button-new-with-label (car bt)))
		      button-template)))

    (gtk-table-set-row-spacings table 5)
    (gtk-table-set-col-spacings table 5)
    (gtk-container-border-width table 0)
    (gtk-box-pack-start outer-box table #t #t 0)
    (gtk-widget-show table)
    
    (for-each 
     (lambda (b bt)
       (let ((x (cadr bt))
	     (y (caddr bt))
	     (clicked (list-ref buttons (cadddr bt))))
	 (gtk-signal-connect b "clicked" 
			     (lambda () (toggle-button clicked)))
	 ;; (gtk-signal-connect b "enter_notify_event"
	 ;;		     (lambda (ev) (pk 'enter ev) #f))
	 (gtk-table-attach table b x (+ 1 x) y (+ 1 y)
			   '(expand fill) '(expand fill)
			   0 0)
	 (gtk-widget-show b)))
     buttons button-template)

    '()))

(define (make-three-buttons outer-box maker)
  (map (lambda (l)
	 (let ((b (maker l)))
	   (gtk-box-pack-start outer-box b #t #t 0)
	   (gtk-widget-show b)
	   b))
       '("button1" "button2" "button3")))
  
(define-test create-toggle-buttons "toggle buttons"
  (make-three-buttons outer-box gtk-toggle-button-new-with-label)
  '())

(define-test create-check-buttons "check buttons"
  (make-three-buttons outer-box gtk-check-button-new-with-label)
  '())

(define-test create-radio-buttons "radio buttons"
  (let ((group #f))
    (define (radio-maker label)
      (set! group (gtk-radio-button-new-with-label group label))
      group)
    (make-three-buttons outer-box radio-maker))
  '())

(define-test create-reparent "reparent"
  (let ((hbox (gtk-hbox-new #f 5))
	(label (gtk-label-new "Hello World")))
    (define (make-frame title init-label)
      (let ((frame (gtk-frame-new title))
	    (box (gtk-vbox-new #f 5))
	    (button (gtk-button-new-with-label "switch")))
	(gtk-box-pack-start hbox frame #t #t 0)
	(gtk-widget-show frame)
	(gtk-container-border-width box 5)
	(gtk-container-add frame box)
	(gtk-widget-show box)
	(gtk-signal-connect button "clicked" 
			    (lambda () (gtk-widget-reparent label box)))
	(gtk-box-pack-start box button #f #t 0)
	(gtk-widget-show button)
	(cond (init-label
	       (gtk-box-pack-start box label #f #t 0)
	       (gtk-widget-show label)))))

    (gtk-box-pack-start outer-box hbox #t #t 0)
    (gtk-widget-show hbox)
    (make-frame "Frame 1" #t)
    (make-frame "Frame 2" #f)
    '()))

(define-test create-pixmap "pixmap"
  (let* ((button (gtk-button-new))
	 (pixmap (gtk-pixmap-new "test.xpm" button))
	 (label (gtk-label-new "Pixmap test"))
	 (hbox (gtk-hbox-new #f 0)))
    (gtk-box-pack-start outer-box button #f #f 0)
    (gtk-widget-show button)
    (gtk-container-border-width hbox 2)
    (gtk-container-add hbox pixmap)
    (gtk-container-add hbox label)
    (gtk-container-add button hbox)
    (gtk-widget-show pixmap)
    (gtk-widget-show label)
    (gtk-widget-show hbox)
    '()))

(define tooltips #f)

(define-test create-tooltips "tooltips"
  (if (not tooltips)
      (set! tooltips (gtk-tooltips-new)))
  (let ((buttons  (make-three-buttons outer-box
				      gtk-toggle-button-new-with-label)))
    (for-each (lambda (b tip)
		(gtk-tooltips-set-tip tooltips b tip ""))
	      buttons
	      '("This is button 1"
		"This is button 2"
		"This is button 3. This is also a really long tooltip which probably won't fit on a single line and will therefore need to be wrapped. Hopefully the wrapping will work correctly."))
    '()))
	
(define (make-menu depth)
  (if (< depth 1)
      #f
      (let ((menu (gtk-menu-new))
	    (submenu (make-menu (1- depth)))
	    (group #f))
	(do ((i 0 (1+ i))
	     (j 1 (1+ j)))
	    ((= i 6))
	  (let ((menuitem (gtk-radio-menu-item-new-with-label group
			   (string-append "item " (number->string depth)
					  " - " (number->string j)))))
	    (set! group menuitem)
	    (gtk-menu-append menu menuitem)
	    (gtk-widget-show menuitem)
	    (if submenu
		(gtk-menu-item-set-submenu menuitem submenu))))
	menu)))
      
(define-test create-menus "menus"
  (let ((menubar (gtk-menu-bar-new))
	(menu (make-menu 2))
	(optionmenu (gtk-option-menu-new)))
    (gtk-box-pack-start outer-box menubar #f #t 0)
    (gtk-widget-show menubar)
    (for-each (lambda (label)
		(let ((menuitem (gtk-menu-item-new-with-label label)))
		  (gtk-menu-item-set-submenu menuitem menu)
		  (gtk-menu-bar-append menubar menuitem)
		  (gtk-widget-show menuitem)))
	      '("test" "foo" "bar"))
    (gtk-option-menu-set-menu optionmenu (make-menu 1))
    (gtk-option-menu-set-history optionmenu 4)
    (gtk-box-pack-start outer-box optionmenu #t #t 0)
    (gtk-widget-show optionmenu)
    '()))

(define create-scrolled-windows #f)
(define create-entry #f)

(define-test create-list "list"
  (let ((list-items '("hello" 
		      "world"
		      "blah"
		      "foo"
		      "bar"
		      "argh"
		      "spencer"
		      "is a"
		      "wussy"
		      "programmer"))
	(scrolled-win (gtk-scrolled-window-new))
	(lyst (gtk-list-new))
	(add (gtk-button-new-with-label "add"))
	(remove (gtk-button-new-with-label "remove")))
	     
    (gtk-scrolled-window-set-policy scrolled-win 'automatic 'automatic)
    (gtk-box-pack-start outer-box scrolled-win #t #t 0)
    (gtk-widget-show scrolled-win)

    (gtk-list-set-selection-mode lyst 'multiple)
    (gtk-list-set-selection-mode lyst 'browse)
    (gtk-container-add scrolled-win lyst)
    (gtk-widget-show lyst)

    (for-each (lambda (i)
		(let ((list-item (gtk-list-item-new-with-label i)))
		  (gtk-container-add lyst list-item)
		  (gtk-widget-show list-item)))
	      list-items)
    
    (gtk-signal-connect add "clicked" (lambda () (pk 'add)))
    (gtk-box-pack-start outer-box add #f #t 0)
    (gtk-widget-show add)

    (gtk-signal-connect remove "clicked" (lambda () (pk 'remove)))
    (gtk-box-pack-start outer-box remove #f #t 0)
    (gtk-widget-show remove)

    (gtk-signal-connect lyst "select_child" 
			(lambda (child)
			  (pk 'selected child
			      (gtk-list-child-position lyst child))))

    ; unselect_child is never emited...
    (gtk-signal-connect lyst "unselect_child" (lambda (child)
					      (pk 'unselected child)))

    '()))

(define-dialog-test create-color-selection
  (let ((window #f))
    (gtk-preview-set-install-cmap #t)
    (gtk-widget-push-visual (gtk-preview-get-visual))
    (gtk-widget-push-colormap (gtk-preview-get-cmap))
    
    (set! window (gtk-color-selection-dialog-new
		  "color selection dialog"))
    ;; (let ((colorsel (gtk-color-selection-dialog-colorsel window)))
    ;; (gtk-signal-connect colorsel "color_changed" 
    ;;		  (lambda () (pk 'changed))))
    (gtk-signal-connect
     (gtk-color-selection-dialog-ok-button window)
     "clicked" (lambda () (pk 'ok)))
    (gtk-signal-connect 
     (gtk-color-selection-dialog-cancel-button window)
     "clicked" (lambda () (gtk-widget-destroy window)))
    
    (gtk-widget-pop-colormap)
    (gtk-widget-pop-visual)
    window))

(define create-file-selection #f)
(define create-dialog #f)

(define-test create-range-controls "range controls"
  (let* ((adjustment (gtk-adjustment-new 0.0 0.0 101.0 0.1 1.0 1.0))
	 (scale (gtk-hscale-new adjustment))
	 (scrollbar (gtk-hscrollbar-new adjustment)))

    (gtk-widget-set-usize scale 150 30)
    (gtk-range-set-update-policy scale 'delayed)
    (gtk-scale-set-digits scale 2)
    (gtk-scale-set-draw-value scale #t)
    (gtk-box-pack-start outer-box scale #t #t 0)
    (gtk-widget-show scale)

    (gtk-range-set-update-policy scrollbar 'continuous)
    (gtk-box-pack-start outer-box scrollbar #t #t 0)
    (gtk-widget-show scrollbar)

    '()))

(define create-rulers #f)

(define-test create-text "text"
  (let* ((t (gtk-text-new #f #f)))
    (gtk-box-pack-start outer-box t #t #t 0)
    (gtk-widget-show t)

    (gtk-widget-realize t)
    (gtk-text-insert t #f "white" "green" "Hallo!\n" -1)
    (gtk-text-insert t #f "yellow" "red" "Huhu!\n" -1)
    (gtk-text-thaw t)

    '()))

(define-test create-notebook "notebook" 
  (let ((notebook (gtk-notebook-new)))
    (gtk-notebook-set-tab-pos notebook 'top)
    (gtk-box-pack-start outer-box notebook #t #t 0)
    (gtk-widget-show notebook)

    (do ((i 1 (1+ i)))
	((= i 6))
      (let* ((text (string-append "Page " (number->string i)))
	     (frame (gtk-frame-new text))
	     (label1 (gtk-label-new text))
	     (label2 (gtk-label-new text)))
	(gtk-container-border-width frame 10)
	(gtk-widget-set-usize frame 200 150)
	(gtk-widget-show frame)
	(gtk-container-add frame label1)
	(gtk-widget-show label1)
	(gtk-notebook-append-page notebook frame label2)))

    `(("next" ,(lambda () 
		 (gtk-notebook-next-page notebook)))
      ("prev" ,(lambda ()
		 (gtk-notebook-prev-page notebook)))
      ("rotate" ,(lambda ()
		   (gtk-notebook-set-tab-pos 
		    notebook
		    (cdr (assq (gtk-notebook-tab-pos notebook)
			       '((top . left)
				 (left . bottom)
				 (bottom . right)
				 (right . top))))))))))

(define-test create-progress-bar "progress"
  (let* ((timer #f)
	 (label (gtk-label-new "progress..."))
	 (pbar (gtk-progress-bar-new)))
    
    (define (timeout)
      (let ((val (gtk-progress-bar-percentage pbar)))
	(gtk-progress-bar-update pbar (+ 0.02 (if (>= val 1) 0 val)))
	#t))

    (gtk-signal-connect pbar "destroy" (lambda () 
					 (gtk-timeout-remove timer)
					 (set! timer #f)))

    (gtk-misc-set-alignment label 0 0.5)
    (gtk-box-pack-start outer-box label #f #t 0)
    (gtk-widget-show label)
    (gtk-widget-set-usize pbar 200 20)
    (gtk-box-pack-start outer-box pbar #t #t 0)
    (gtk-widget-show pbar)

    (set! timer (gtk-timeout-add 100 timeout))
    '()))

(define create-color-preview #f)
(define create-gray-preview #f)
(define create-curve #f)

(define (make-counter outer-box adder remover)
  (let ((tag #f)
	(count 0)
	(label (gtk-label-new "")))
    (define (set-label)
      (gtk-label-set label
		     (string-append "count: " (number->string count))))
    (define (tick)
      (set! count (1+ count))
      (set-label))
    (define (start)
      (if (not tag) (set! tag (adder tick))))
    (define (stop)
      (if tag (remover tag))
      (set! tag #f))
    (define (shot)
      (start)
      (gtk-timeout-add 1001 (lambda () (stop) #f)))
    (gtk-signal-connect label "destroy" stop)

    (set-label)
    (gtk-misc-set-padding label 10 10)
    (gtk-box-pack-start outer-box label #t #t 0)
    (gtk-widget-show label)

    `(("shot"  ,shot)
      ("start" ,start)
      ("stop"  ,stop))))

(define-test create-timeout-test "Timeout Test"
  (make-counter outer-box
		(lambda (p) (gtk-timeout-add 100 p)) gtk-timeout-remove))

(define-test create-idle-test "Idle Test"
  (make-counter outer-box gtk-idle-add gtk-idle-remove))

(define create-test #f)

(define (yes-or-no? question)
  (let ((d (gtk-dialog-new))
	(l (gtk-label-new question))
	(y (gtk-button-new-with-label "Yes"))
	(n (gtk-button-new-with-label "No"))
	(answer #f))

    (gtk-window-set-title d "Yes or No?")
    (gtk-misc-set-alignment l 0.5 0.5)
    (gtk-box-pack-start (gtk-dialog-vbox d) l #f #f 0)
    (gtk-box-pack-start (gtk-dialog-action-area d) y #t #t 3)
    (gtk-box-pack-start (gtk-dialog-action-area d) n #t #t 3)
    (gtk-widget-show-all d)

    (gtk-signal-connect d "delete_event"
			(lambda (ev)
			  (gtk-main-quit) #t))
    (gtk-signal-connect n "clicked"
			(lambda () 
			  (gtk-widget-destroy d)
			  (gtk-main-quit)))
    (gtk-signal-connect y "clicked"
			(lambda () 
			  (set! answer #t)
			  (gtk-widget-destroy d)
			  (gtk-main-quit)))

    (gtk-grab-add d)
    (gtk-main)
    (gtk-grab-remove d)

    answer))

(define (create-main-window)
  (define buttons
    `(("buttons" ,create-buttons)
      ("toggle buttons" ,create-toggle-buttons)
      ("check buttons" ,create-check-buttons)
      ("radio buttons" ,create-radio-buttons)
      ("reparent" ,create-reparent)
      ("pixmap" ,create-pixmap)
      ("tooltips" ,create-tooltips)
      ("menus" ,create-menus)
      ("scrolled windows" ,create-scrolled-windows)
      ("drawing areas" #f)
      ("entry" ,create-entry)
      ("list" ,create-list)
      ("color selection" ,create-color-selection)
      ("file selection" ,create-file-selection)
      ("dialog" ,create-dialog)
      ("miscellaneous" #f)
      ("range controls" ,create-range-controls)
      ("rulers" ,create-rulers)
      ("text" ,create-text)
      ("notebook" ,create-notebook)
      ("progress bar" ,create-progress-bar)
      ("preview color" ,create-color-preview)
      ("preview gray" ,create-gray-preview)
      ("curve" ,create-curve)
      ("test timeout" ,create-timeout-test)
      ("test idle" ,create-idle-test)
      ("test" ,create-test)))

  (let ((window (gtk-window-new 'toplevel))
	(box1   (gtk-vbox-new #f 0))
	(box2   (gtk-vbox-new #f 0))
	(separator (gtk-hseparator-new))
	(box3   (gtk-vbox-new #f 10))
	(gc-but (gtk-button-new-with-label "gc"))
	(button (gtk-button-new-with-label "close")))
    
    (gtk-widget-set-name window "main window")
    (gtk-widget-set-uposition window 20 20)
    (gtk-container-add window box1)
    (gtk-container-border-width box2 10)
    (gtk-box-pack-start box1 box2 #t #t 0)
    
    (for-each (lambda (b)
		(let ((button (gtk-button-new-with-label (car b))))
		  (if (cadr b)
		      (gtk-signal-connect button "clicked" (cadr b))
		      (gtk-widget-set-sensitive button #f))
		  (gtk-box-pack-start box2 button #t #t 0)))
	      buttons)
       
    (gtk-box-pack-start box1 separator #f #t 0)
    (gtk-container-border-width box3 10)
    (gtk-box-pack-start box1 box3 #f #t 0)
    (gtk-signal-connect gc-but "clicked" gc)
    (gtk-box-pack-start box3 gc-but #t #t 0)
    (gtk-signal-connect button "clicked" gtk-exit)
    (gtk-box-pack-start box3 button #t #t 0)
    (gtk-widget-set-flags button '(can-default))
    (gtk-widget-grab-default button)

    (gtk-signal-connect window "delete_event" 
			(lambda (ev) 
			  (if (yes-or-no? "Quit?")
			      (if (gtk-standalone?) 
				  (gtk-exit)
				  (gtk-widget-destroy window))
			      #t)))
    
    (gtk-widget-show-all window)))

(define break-iteration #f)
(define (breakable-iteration)
  (with-return
   (set! break-iteration return)
   (gtk-main-iteration)))

(gtk-rc-parse "testgtkrc")
(create-main-window)

(if (gtk-standalone?) (gtk-main))

; Local Variables:
; mode: scheme
; End:
